<?php
namespace BDE\BdegmbhWebservices\Hooks;

use BDE\BdegmbhWebservices\Internal\HttpHeaderAnalyseTools;
use BDE\BdegmbhWebservices\Internal\UriTemplateParser;
use \TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * Class Methods
 * @package BDE\BdegmbhWebservices\Hooks
 */
class Methods implements \TYPO3\CMS\Core\SingletonInterface {
	
	private $services = array();
	private $pathAnnotation = "Path";
	private $queryParamAnnotation = "QueryParam";
	private $producesAnnotation = "Produces";
	private $httpMethods = array("POST", "GET");

	/**
	 * Methods constructor.
	 */
	public function __construct() {

		if (is_array($GLOBALS['TYPO3_CONF_VARS']['EXTCONF']['bdegmbh_webservices']['services'])) {
			foreach ($GLOBALS['TYPO3_CONF_VARS']['EXTCONF']['bdegmbh_webservices']['services'] as $classRef) {
				$obj = \TYPO3\CMS\Core\Utility\GeneralUtility::getUserObj($classRef);
				$classReflection = GeneralUtility::makeInstance('TYPO3\CMS\Extbase\Reflection\ClassReflection',
					get_class($obj));
				
				// Check, if @path annotation exist 
				if ($classReflection->isTaggedWith($this->pathAnnotation) &&
					$classReflection->getTagValues($this->pathAnnotation)[0] !== "") {

					$pathValue = $classReflection->getTagValues($this->pathAnnotation)[0];
					$classMethod = $classReflection->getMethods();

					$classProduceMimeType = $classReflection->isTaggedWith($this->producesAnnotation) ?
							$classReflection->getTagValues($this->producesAnnotation) : array("text/html");
										
					// Check each method if annotaion @Path exist
					foreach ($classMethod as $method) {
						
						if ($method->isTaggedWith($this->pathAnnotation) &&
							$method->getTagValues($this->pathAnnotation)[0] !== "") {

							// Create a template parse and save the object
							$parser = new UriTemplateParser($method, $method->getTagValues($this->pathAnnotation)[0]);
							$parser->setObject($obj);

							$queryParams = $method->isTaggedWith($this->queryParamAnnotation) ?
								$method->getTagValues($this->queryParamAnnotation) : array();

							// get all MIME type from header
							$produceMimeTypes = explode(",", preg_replace('/\s+/', '', implode(",",
								$method->isTaggedWith($this->producesAnnotation) ?
								$method->getTagValues($this->producesAnnotation) : $classProduceMimeType)));
							$parser->setProduceMimeType($produceMimeTypes);

							// Check if all parameter match placeholders in URL or in annotation QueryParam
							foreach($method->getParameters() as $parameter) {
								 if (!in_array($parameter->getName(), $parser->getNames()) &&
									 !in_array($parameter->getName(), $queryParams)) {
									 continue 2;
								 }
							}

							// Method must be tagged with a HTTP-Method
							foreach($this->httpMethods as $httpMethod) {
								if ($method->isTaggedWith($httpMethod)) {
									$this->services[$pathValue][$httpMethod][] = $parser;
								}
							}
						}
					}
				}
			}
		}
	}

	/**
	 * 
	 * @param unknown $params
	 */
	public function decodeSpURLPreProc($params, $pObj)
	{

		$speakingURIpath = rtrim(parse_url($params["URL"], PHP_URL_PATH), "/");
		$urlpath = explode("/", $speakingURIpath);

		$serviceRoot = $GLOBALS['TYPO3_CONF_VARS']['EXTCONF']['bdegmbh_webservices']['serviceRoot'];

		if ($urlpath[0] !== $serviceRoot || sizeof($urlpath) <= 2) {
			return;
		}

		if (!array_key_exists("/" . $urlpath[1], $this->services)) {
			return;
		}

		if (!array_key_exists($_SERVER['REQUEST_METHOD'], $this->services["/" . $urlpath[1]])) {
			return;
		}

		$accept = array_key_exists('HTTP_ACCEPT', $_SERVER) ?
			HttpHeaderAnalyseTools::analyseAccept($_SERVER["HTTP_ACCEPT"]) : array();

		$parserList = $this->services["/" . $urlpath[1]][$_SERVER['REQUEST_METHOD']];
		$URIpath = "/".implode("/", array_slice ($urlpath , 2));

		foreach ($parserList as $parser) {

			if (!preg_match($parser->getPattern(), $URIpath, $matches))
				continue;

			if (($acceptedMimeType = $parser->getAcceptedMimeType($accept)) === null) {
				continue;
			}

			$arg_list = array();

			switch($_SERVER['REQUEST_METHOD']) {
				case 'GET':
					$request = &$_GET;
					break;
				case 'POST':
					$request = &$_POST;
					break;
			}

			// Fill the argument list with requested values
			foreach($parser->getMethod()->getParameters() as $parameter) {
				$pos = array_search($parameter->getName(), $parser->getNames());

				if (!($pos === FALSE)) {
					$arg_list[] = $matches[$pos + 1];
				} elseif (array_key_exists($parameter->getName(), $request)) {
					$arg_list[] = $request[$parameter->getName()];
				} else {
					continue 2;
				}
			}

			// Call the method
			$retval = $parser->getMethod()->invokeArgs($parser->getObject(), $arg_list);
			$mimeType = explode("/", $acceptedMimeType);
			$producerName = 'BDE\BdegmbhWebservices\Producer\\'.ucfirst($mimeType[0]).ucfirst($mimeType[1])."Producer";
			header('Content-Type: '.$acceptedMimeType);

			// check if producer exist
			if (class_exists($producerName) && !is_string($retval)) {
				$producer = GeneralUtility::makeInstance($producerName);
				echo $producer->produce($retval);
			} else
				echo $retval;

			exit();
		}
	}
}