<?php
namespace BDE\BdegmbhWebservices\Internal;

use BDE\BdegmbhWebservices\Exception\IllegalArgumentException;
/**
 * This file was adapted to PHP from org.glassfish.jersey.uri.internal.UriTemplateParser
 * which is part of open source Jersey RESTful Web Services framework https://jersey.java.net/
 *
 * Date: 16.12.2015
 * Time: 18:26
 */
class UriTemplateParser
{
    private $RESERVED_REGEX_CHARACTERS = array('.', '^', '&', '!', '?', '-', ':', '<', '(', '[', '$', '=',
        ')', ']', ',', '>', '*', '+', '|');

    private $HEX_CHARACTERS = array('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', 'A', 'B', 'C', 'D', 'E', 'F',
        'a', 'b', 'c', 'd', 'e', 'f');
    private $TEMPLATE_VALUE_PATTERN = "[^/]+";
    private $template;
    private $method;
    private $literalCharactersBuffer = array();
    private $literalCharacters = 0;
    private $normalizedTemplate = "";
    private $regex = "";
    private $HexToUpperCaseRegex;
    private $skipGroup;
    private $names = array();
    private $numOfExplicitRegexes = 0;
    private $nameToPattern = array();
    private $obj;
    private $produces;

    /**
     * UriTemplateParser constructor.
     * @param $template
     */
    public function __construct($method, $template) {
        $this->template = $template;
        $this->method = $method;

        for($i = 0; $i < 128; $i++)
            $this->HexToUpperCaseRegex[$i] = chr($i);

        for ($c = 'a'; $c <= 'f'; $c++) {
            $this->HexToUpperCaseRegex[$c] = "[" . $c . chr(ord($c) - ord('a') + ord('A')) . "]";
        }

        for ($c = 'A'; $c <= 'F'; $c++) {
            $this->HexToUpperCaseRegex[$c] =  "[" . chr(ord($c) - ord('A') + ord('a')) . $c . "]";
        }

        $this->parse(new CharacterIterator($template));
    }

    /**
     * @return mixed
     */
    public function getMethod() {
        return $this->method;
    }

    /**
     * @param $produces
     */
    public function setProduceMimeType($produces) {
        $this->produces = $produces;
    }

    /**
     * @return mixed
     */
    public function getProduceMimeType() {
        return $this->produces;
    }

    /**
     * @param $mimeTypes
     * @return null
     */
    public function getAcceptedMimeType($mimeTypes) {

        foreach($mimeTypes as $mimeType) {
            $pattern = $this->createRegExFromString($mimeType["type"]);

            foreach($this->produces as $produceType) {
                if (preg_match($pattern, $produceType))
                    return $produceType;
            }
        }

        return null;
    }

    /**
     * @param $string
     * @return string
     */
    private function createRegExFromString($string) {
        return "/".str_replace("*", "(.*)", str_replace("/", "\/", $string))."/";
    }

    /**
     * @param $obj
     */
    public function setObject($obj) {
        $this->obj = $obj;
    }

    /**
     * @return mixed
     */
    public function getObject() {
        return $this->obj;
    }

    /**
     * @return array
     */
    public function getNames() {
        return $this->names;
    }

    /**
     * @return string
     */
    public function getPattern() {
        return "/^".$this->regex."$/";
    }

    /**
     * @param CharacterIterator $ci
     */
    private function parse(CharacterIterator &$ci) {

        while ($ci->hasNext()) {
            $c = $ci->next();

            if ($c == '{') {
                $this->processLiteralCharacters();
                $this->skipGroup = $this->parseName($ci, $this->skipGroup);
            } else {
                $this->literalCharactersBuffer[] = $c;
            }
        }

        $this->processLiteralCharacters();
    }

    /**
     * @param CharacterIterator $ci
     * @param $skipGroup
     */
    private function parseName(CharacterIterator &$ci, $skipGroup) {
        $c = $this->consumeWhiteSpace($ci);
        $paramType = 'p';
        $nameBuffer = "";

        if ($c == '?' || $c == ';') {
            $paramType = $c;
            $c = $ci->next();
        }

        if (ctype_alnum($c) || $c == '_') {
            $nameBuffer[] = $c;
        } else {
            throw new IllegalArgumentException();
        }

        $nameRegexString = "";

        while (true) {
            $c = $ci->next();

            if (ctype_alnum($c) || $c == '_' || $c == '-' || $c == '.') {
                $nameBuffer[] = $c;
            } else if ($c == ',' && $paramType != 'p') {
                $nameBuffer[] = $c;
            } else if ($c == ':' && $paramType == 'p') {
                $nameRegexString = $this->parseRegex($ci);
                break;
            } else if ($c == '}') {
                break;
            } else if ($c == ' ') {
                $c = $this->consumeWhiteSpace($ci);

                if ($c == ':') {
                    $nameRegexString = $this->parseRegex($ci);
                    break;
                } else if ($c == '}') {
                    break;
                } else {
                    throw new IllegalArgumentException();
                }

            } else {
                throw new IllegalArgumentException();
            }
        }

        $name = implode($nameBuffer);
        $namePattern = "";

        if ($paramType == '?' || $paramType == ';') {
            $subNames = preg_split(",\\s?", $name);
            $regexBuilder = $paramType == '?' ? "\\?" : ";";
            $separator = $paramType == '?' ? "\\&" : ";/\\?";

            $first = true;

            $regexBuilder .= "(";

            foreach($subNames as $subName) {
                $regexBuilder .= "(&?" . $subName . "(=([^" . $separator . "]*))?)";

                if (!$first) {
                    $regexBuilder .= "|";
                }

                $this->names[]  = $subName;
                $first = false;
            }

            $skipGroup = 1;
            $regexBuilder .= ")*";
            $namePattern = $this->correctSlashes($regexBuilder);
            $name = $paramType . $name;
        } else {
            $this->names[] = $name;

            if (!strlen($nameRegexString)) {
                $this->numOfExplicitRegexes++;
            }

            $namePattern = $this->correctSlashes(((strlen($nameRegexString) == 0)
                ? $this->TEMPLATE_VALUE_PATTERN : $nameRegexString));

            if (array_key_exists($name, $this->nameToPattern)) {
                if ($this->nameToPattern[$name] != $namePattern) {
                    throw new IllegalArgumentException();
                }
            } else {
                $this->nameToPattern[$name] = $namePattern;
            }

            $returnValue = preg_match_all($namePattern, '', $matches);
            $skipGroup = $returnValue;
        }

        $this->regex .= '(' . $namePattern .')';

        return $skipGroup;
    }

    /**
     * @param $regex
     * @return mixed
     */
    private function correctSlashes($regex) {
        return str_replace("/", "\\/", $regex);
    }


    /**
     * @param CharacterIterator $ci
     * @return null|string
     */
    private function consumeWhiteSpace(CharacterIterator &$ci) {
        $c = "";

        do {
            $c = $ci->next();
        } while (ctype_space($c));

        return $c;
    }

    /**
     * @param CharacterIterator $ci
     */
    private function parseRegex(CharacterIterator &$ci) {
        $regexBuffer = "";
        $braceCount = 1;

        while (true) {
            $c = $ci->next();

            if ($c == '{') {
                $braceCount++;
            } else if ($c == '}') {
                $braceCount--;

                if ($braceCount == 0) {
                    break;
                }
            }

            $regexBuffer .= $c;
        }

        return $regexBuffer;
    }


    /**
     *
     */
    private function processLiteralCharacters() {

        if (sizeof($this->literalCharactersBuffer) > 0) {
            $this->literalCharacters += sizeof($this->literalCharactersBuffer);
            $s = implode($this->literalCharactersBuffer);
            $this->normalizedTemplate .= $s;

            for ($i = 0; $i < strlen($s); $i++) {
                $char = $s[$i];

                if (in_array($char, $this->RESERVED_REGEX_CHARACTERS)) {
                    $this->regex .= "\\";
                    $this->regex .= $char;
                } elseif ($char == "%") {
                    $c1 = $s[$i + 1];
                    $c2 = $s[$i + 2];

                    if ($this->isHexCharacter($c1) && $this->isHexCharacter($c2)) {
                        $this->regex .= "%".$this->HexToUpperCaseRegex[$c1].$this->HexToUpperCaseRegex[$c2];
                    }
                } else {
                    $this->regex .= $this->correctSlashes($char);
                }
            }

            $this->literalCharactersBuffer = array();
        }
    }

    /**
     * @param $c
     * @return bool
     */
    private function isHexCharacter($c) {
        return ((ord($c) < 128 && in_array($c, $this->HEX_CHARACTERS)));
    }
}