<?php

namespace BDE\BdegmbhWebservices\Producer;

use \TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * Class XMLProducer
 * @package BDE\BdegmbhWebservices\Internal
 */
class ApplicationXmlProducer implements Producer
{
    private $xml;
    private $xmlAnnotation = array('@XmlTransient' => 'XmlTransient', '@XmlAttribute' => 'XmlAttribute');

    /**
     * XMLProducer constructor.
     */
    public function __construct()
    {
        $this->xml = new \XMLWriter();
        $this->xml->openMemory();
    }

    /**
     * @param $arg
     * @return string
     */
    public function produce($arg)
    {
        $this->generateXml($arg);
        return $this->xml->flush();
    }


    /**
     * @param $arg
     */
    private function generateXml($arg) {

        if (is_object($arg)) {
            $classReflection = GeneralUtility::makeInstance('TYPO3\CMS\Extbase\Reflection\ClassReflection',
                get_class($arg));

            $node_block = strtolower(get_class($arg));
            $this->xml->startElement($node_block);
            $properties = $classReflection->getProperties();

            // Check all class properties if them are marked with @XmlAttribute
            foreach($properties as $property) {

                // if property is tagged with @XmlTransient ignore it
                if ($property->isTaggedWith($this->xmlAnnotation["@XmlTransient"]))
                    continue;

                if ($property->isTaggedWith($this->xmlAnnotation["@XmlAttribute"])) {
                    $this->xml->startAttribute(strtolower($property->getName()));
                    $this->xml->text($property->getValue($arg));
                    $this->xml->endAttribute();
                }
            }

            foreach($properties as $property) {

                // if property is tagged with @XmlTransient ignore it
                if ($property->isTaggedWith($this->xmlAnnotation["@XmlTransient"]) ||
                    $property->isTaggedWith($this->xmlAnnotation["@XmlAttribute"]))
                    continue;

                $obj = $property->getValue($arg);

                if (is_object($obj))
                    $this->generateXml($obj);
                else {
                    $this->xml->startElement(strtolower($property->getName()));
                    $this->generateXml($obj);
                    $this->xml->endElement();
                }
            }

            $this->xml->endElement();

        } else if(is_array($arg)) {

            $this->xml->startElement("array");

            foreach($arg as $key => $value) {
                $this->xml->startElement(is_numeric($key) ? "node_".$key : $key);
                $this->generateXml($value);
                $this->xml->endElement();
            }

            $this->xml->endElement();

        } else {
            $this->xml->text(htmlspecialchars($arg, ENT_QUOTES));
        }
    }
}